// xDBSearch Package  1998, Deks
// Version info in readme.txt and help
//
// http://deks.webjump.com/
// E-mail: Deks@online.nsk.su
unit uxDBSearch;

interface

uses
  Classes, Controls, ExtCtrls, Menus, DB, Forms, SysUtils;

type
  TxDBSearch = class;

  TExDBSearchError = class(Exception);

  // ==================
  // TxDBSearchField
  // ==================
  TxDBSearchField = class(TComponent)
  protected
    FField: TField;
    FDBSearch: TxDBSearch;
    FFieldCaption: String;

    procedure SetDBSearch(Value: TxDBSearch);

    procedure SetParentComponent(Value: TComponent); override;
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function HasParent: Boolean; override;
    function GetParentComponent: TComponent; override;

    property DBSearch: TxDBSearch read FDBSearch write SetDBSearch;
  published
    property Field: TField read FField write FField;
    property FieldCaption: String read FFieldCaption write FFieldCaption;
  end;

  // ==================
  // TxDBSearch
  // ==================
  TxDBSearch = class(TComponent)
  protected
    FDBSearchFields: TList;
    FDataSet: TDataSet;
    FAutoLocate: Boolean;
    FSearchForm: TForm;
    FModified: Boolean;
    FDialogTitle: String;

    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;

    procedure SetDBSearchFields( Value: TList);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Add(DBSearchField: TxDBSearchField);
    procedure Remove(DBSearchField: TxDBSearchField);
    procedure Clear;

    procedure Execute;

    property Modified: Boolean read FModified write FModified;
  published
    property AutoLocate: Boolean read FAutoLocate write FAutoLocate default True;
    property DataSet: TDataSet read FDataSet write FDataSet;
    property DBSearchFields: TList read FDBSearchFields write SetDBSearchFields;
    property DialogTitle: String read FDialogTitle write FDialogTitle;
  end;

implementation

uses StdCtrls, TypInfo, Windows, uxDBSearchForm, uxDBSearchConstants;

var
  Registered: Boolean = False;

// ============================================================================
// TxDBSearchField
// ============================================================================
constructor TxDBSearchField.Create(AOwner: TComponent);
begin
  // .

  //   :
  inherited Create(AOwner);

  //   :
  FField := nil;
end;

// ----------------------------------------------------------------------------
destructor TxDBSearchField.Destroy;
begin
  // .

  //        :
  if FDBSearch <> nil then
    FDBSearch.Remove(Self);

  //     :
  FDBSearch := nil;

  //   :
  inherited Destroy;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearchField.Notification(AComponent: TComponent; AOperation: TOperation);
begin
  //   -   .

  //   .
  inherited Notification(AComponent, AOperation);

  //      (  ,  
  //    ):
  if AOperation = opRemove then begin
    if AComponent = Field then Field := nil;
  end;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearchField.SetDBSearch(Value: TxDBSearch);
begin
  //   SetDBSearch.

  // ;   ,      
  // :
  if Value = FDBSearch then Exit;

  //     :
  if FDBSearch <> nil then
    FDBSearch.Remove(Self);

  //   :
  FDBSearch := Value;

  //     :
  if FDBSearch <> nil then
    FDBSearch.Add(Self);
end;

// ----------------------------------------------------------------------------
function TxDBSearchField.HasParent: Boolean;
begin
  //     ?  !
  Result := True;
end;

// ----------------------------------------------------------------------------
function TxDBSearchField.GetParentComponent: TComponent;
begin
  //    -    ...    !
  Result := FDBSearch;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearchField.SetParentComponent(Value: TComponent);
begin
  //    (  
  //  GetParentcomponent)
  if FDBSearch <> nil then
    FDBSearch.Remove(Self);
  if (Value <> nil) and (Value is TxDBSearch) then
    DBSearch := TxDBSearch(Value);
end;

// ============================================================================
// TxDBSearch
// ============================================================================
constructor TxDBSearch.Create(AOwner: TComponent);
begin
  // .

  //   :
  inherited Create(AOwner);

  //     -:
  FDataSet := nil;
  FAutoLocate := True;
  FSearchForm := nil;
  FModified := False;
  FDialogTitle := SxDBS_DefaultDialogTitle;
     
  //   :
  FDBSearchFields := TList.Create;

  //     :
  if not Registered then begin
    RegisterClasses([TxDBSearchField]);
    Registered := True;
  end;
end;

// ----------------------------------------------------------------------------
destructor TxDBSearch.Destroy;
begin
  // .

  //   :
  Clear;

  //     :
  if FDBSearchFields <> nil then
    FDBSearchFields.Free;

  //   :
  inherited Destroy;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearch.Loaded;
//var Loading: Boolean;
begin
//  Loading := csLoading in ComponentState;
  inherited Loaded;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearch.Notification(AComponent: TComponent; AOperation: TOperation);
begin
  inherited Notification(AComponent, AOperation);
  if AOperation = opRemove then begin
    if AComponent = DataSet then DataSet := nil;
  end;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearch.GetChildren(Proc: TGetChildProc; Root: TComponent);
var I: Integer;
begin
  inherited GetChildren(Proc, Root);
  for I := 0 to FDBSearchFields.Count - 1 do begin
    Proc(TxDBSearchField(FDBSearchFields.Items[I]));
  end;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearch.SetDBSearchFields(Value: TList);
begin
end;

// ----------------------------------------------------------------------------
procedure TxDBSearch.Add(DBSearchField: TxDBSearchField);
var
  i: Integer;
  isFound: Boolean;
begin
  //    .
  if (DBSearchField = nil) or (FDBSearchFields = nil) then Exit;

  isFound := False;
  for i:=0 to FDBSearchFields.Count - 1  do begin
    if FDBSearchFields[i] = DBSearchField then
      isFound := True;
  end;

  if (isFound = False) then begin
    FDBSearchFields.Add(DBSearchField);
    DBSearchField.DBSearch := Self;
  end;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearch.Remove(DBSearchField: TxDBSearchField);
var
  i: Integer;
  isFound: Boolean;
begin
  //    .
  if (DBSearchField.DBSearch <> Self) or (DBSearchField = nil)  then Exit;

  //     :
  isFound := False;
  for i:=0 to FDBSearchFields.Count - 1  do begin
    if FDBSearchFields[i] = DBSearchField then
      isFound := True;
  end;

  //  ,     :
  if isFound = True then begin
    FDBSearchFields.Remove(DBSearchField);
    DBSearchField.DBSearch := nil;
  end;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearch.Clear;
var
  DBSearchField: TxDBSearchField;
begin
  //     :
  while FDBSearchFields.Count > 0 do begin
    DBSearchField := FDBSearchFields.Last;
    Remove(DBSearchField);
    DBSearchField.Free;
  end;
end;

// ----------------------------------------------------------------------------
procedure TxDBSearch.Execute;
var
  aSearchForm: TxDBSearchForm;
  isInitControls: Boolean;
begin
  //    "".

  //     
  isInitControls := FModified;
   
  //      ( ):
  aSearchForm := TxDBSearchForm(FSearchForm);
  if aSearchForm = nil then begin
    //   :
    FSearchForm := TxDBSearchForm.Create(Self);
    aSearchForm := TxDBSearchForm(FSearchForm);

    //  :
    if aSearchForm <> nil then begin
      //     :
      aSearchForm.DBSearch := Self;
      aSearchForm.Caption := FDialogTitle;
      isInitControls := True;
    end;
  end;

  //   ,      :
  if (isInitControls) and (aSearchForm <> nil) then
    aSearchForm.InitControls;
    
  //   :
  if (aSearchForm = nil) or (FDataSet = nil) then Exit;

  //   
  if aSearchForm.ShowModal = mrOK then
    //      :
    if not FDataSet.Locate(aSearchForm.SearchField, aSearchForm.SearchText,
        aSearchForm.SearchOptions)
    then
      raise TExDBSearchError.CreateFmt(SxDBS_ENotFound, [aSearchForm.SearchText]);
//      Application.MessageBox(
//        'Not found', mb_Ok+mb_IconInformation);
end;

end.
