// xDBSearch Package  1998, Deks
// Version info in readme.txt and help
//
// http://deks.webjump.com/
// E-mail: Deks@online.nsk.su
unit uxDBSearchEditor;

interface

uses
  Windows, SysUtils, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DsgnIntf, DsgnWnds, LibIntf, StdCtrls, ExtCtrls, VCLUtils,
  uxDBSearch, Placemnt;

type
  // =========================================
  // TxDBSearchFieldsProperty
  // =========================================
  TxDBSearchFieldsProperty = class(TPropertyEditor)
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure Edit; override;
  end;

  // =========================================
  // TxDBSearchEditor
  // =========================================
  TxDBSearchEditor = class(TComponentEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  // =========================================
  // TFormDBSearchEditor }
  // =========================================
  TFormDBSearchEditor = class(TDesignWindow)
    Grid: TDrawGrid;
    BtnPanel: TPanel;
    CloseBtn: TButton;
    BtnDelete: TButton;
    BtnAdd: TButton;
    FormStorage: TFormStorage;
    PanelOptions: TPanel;
    CbConfirmDelete: TCheckBox;
    BtnClear: TButton;
    procedure FormClose(Sender: TObject; var DBSearchField: TCloseAction);
    procedure GridDrawCell(Sender: TObject; Col, Row: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure GridSelectCell(Sender: TObject; Col, Row: Integer;
      var CanSelect: Boolean);
    procedure CloseBtnClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
    procedure GridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BtnAddClick(Sender: TObject);
    procedure BtnClearClick(Sender: TObject);
  private
    FxComponent: TxDBSearch;
    FDeleting: Boolean;
    procedure SetComponent(Value: TxDBSearch);
    function GetForm: TCustomForm;
    procedure UpdateData;
    function CheckComponent: Boolean;
    procedure SelectxItem(DBSearchField: TxDBSearchField);
    function xItemByRow(Row: Integer): TxDBSearchField;
  protected
    function UniqueName(xComponent: TComponent): string; override;
    procedure Activated; override;
  public
    procedure FormModified; override;
    procedure FormClosed(Form: TCustomForm); override;
    function GetEditState: TEditState; override;
    procedure ComponentDeleted(xComponent: TComponent); override;
    property xComponent: TxDBSearch read FxComponent write SetComponent;
    property OwnerForm: TCustomForm read GetForm;
  end;

var
  FormDBSearchEditor: TFormDBSearchEditor;

implementation

{$R *.DFM}

uses uxAboutDBSearchForm;

// ============================================================================
function FindEditor(xComponent: TxDBSearch): TFormDBSearchEditor;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to Screen.FormCount - 1 do begin
    if Screen.Forms[I] is TFormDBSearchEditor then begin
      if TFormDBSearchEditor(Screen.Forms[I]).xComponent = xComponent then
      begin
        Result := TFormDBSearchEditor(Screen.Forms[I]);
        Break;
      end;
    end;
  end;
end;

// ============================================================================
procedure ShowActionListEditor(Designer: TDesigner; xComponent: TxDBSearch);
var
  Editor: TFormDBSearchEditor;
begin
  if xComponent = nil then Exit;
  Editor := FindEditor(xComponent);
  if Editor <> nil then begin
    Editor.Show;
    if Editor.WindowState = wsMinimized then Editor.WindowState := wsNormal;
  end
  else begin
    Editor := TFormDBSearchEditor.Create(Application);
    try
      Editor.Designer := TFormDesigner(Designer);
      Editor.xComponent := xComponent;
      Editor.Show;
    except
      Editor.Free;
      raise;
    end;
  end;
end;

// ============================================================================
// TxDBSearchFieldsProperty
function TxDBSearchFieldsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TxDBSearchFieldsProperty.GetValue: string;
begin
  Result := 'DBSearchFields';
  //FmtStr(Result, '(%s)', [GetPropType^.Name]);
end;

procedure TxDBSearchFieldsProperty.Edit;
begin
  ShowActionListEditor(Designer, TxDBSearch(GetComponent(0)));
end;

// ============================================================================
// TxDBSearchEditor
procedure TxDBSearchEditor.ExecuteVerb(Index: Integer);
begin
  // Execute menu command.

  // Test arguments and objects, if they are valid:
  if not (Component is TxDBSearch) then Exit;

  // select action by menu index:
  case Index of
    0,1: begin
      // Show about form.
      with TxAboutDBSearchForm.Create(Application) do begin
        try
          ShowModal;
        finally
          Free;
        end;
      end;
    end;

    3: ShowActionListEditor(Designer, TxDBSearch(Component));
// Known bug:
//    1: if (Component<>nil) then
//         (TxDBSearch(Component)).Execute;
  end;
end;

function TxDBSearchEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := 'xDBSearch Package v1.1';
    1: Result := ' 1998, Deks';
    2: Result := '-';
    3: Result := 'xDBSearch Editor ...';
// Known bug: AV error when executing.
//    4: Result := 'xDBSearch Editor ...';
  end;
end;

function TxDBSearchEditor.GetVerbCount: Integer;
begin
  // Return menu item's count.
  Result := 4;
end;

// ============================================================================
// TFormDBSearchEditor
procedure TFormDBSearchEditor.SetComponent(Value: TxDBSearch);
begin
  if FxComponent <> Value then begin
    FxComponent := Value;
    UpdateData;
  end;
end;

function TFormDBSearchEditor.UniqueName(xComponent: TComponent): string;
var
  Temp: string;
begin
  Result := '';

  if (xComponent <> nil)
  then Temp := xComponent.ClassName
  else Temp := 'TComponent';

  if (UpCase(Temp[1]) = 'T') and (Length(Temp) > 1) then
    System.Delete(Temp, 1, 1);

  Result := Designer.UniqueName(Temp);
end;

function TFormDBSearchEditor.GetEditState: TEditState;
begin
  Result := [];
end;

procedure TFormDBSearchEditor.FormClosed(Form: TCustomForm);
begin
  if Form = OwnerForm then Close;
end;

procedure TFormDBSearchEditor.FormModified;
begin
  if not (csDestroying in ComponentState) then UpdateData;
end;

procedure TFormDBSearchEditor.Activated;
begin
  SelectxItem(xItemByRow(Grid.Row - 1));
end;

procedure TFormDBSearchEditor.ComponentDeleted(xComponent: TComponent);
begin
  if xComponent = FxComponent then begin
    FxComponent := nil;
    Close;
  end;
end;

procedure TFormDBSearchEditor.UpdateData;
var
  aCount: Integer;
begin
  //   :

  if CheckComponent then begin
    //   :
    aCount := FxComponent.DBSearchFields.Count;
    if aCount = 0 then begin
      Grid.RowCount := 2;
      Grid.FixedRows := 1;
      SelectxItem(nil);
    end else begin
      Grid.RowCount := aCount+1;
      Grid.FixedRows := 1;
    end;

    //    :
    BtnDelete.Enabled := aCount > 0;

    //  : 
    Grid.Invalidate;
  end;
end;

function TFormDBSearchEditor.GetForm: TCustomForm;
begin
  Result := Designer.Form;
end;

procedure TFormDBSearchEditor.FormClose(Sender: TObject;
  var DBSearchField: TCloseAction);
begin
  DBSearchField := caFree;
end;

function TFormDBSearchEditor.CheckComponent: Boolean;
begin
  Result := (FxComponent <> nil) and (FxComponent.Owner <> nil) and
    (Designer.Form <> nil);
end;

procedure TFormDBSearchEditor.SelectxItem(DBSearchField: TxDBSearchField);
var
  FComponents: TComponentList;
begin
  if CheckComponent and Active then begin
    FComponents := TComponentList.Create;
    if DBSearchField <> nil then
      FComponents.Add(DBSearchField)
    else
      FComponents.Add(FxComponent);
    SetSelection(FComponents);
  end;
end;

function TFormDBSearchEditor.xItemByRow(Row: Integer): TxDBSearchField;
begin
  Result := nil;
  if CheckComponent and (Row >= 0) and
    (Row < FxComponent.DBSearchFields.Count) then
  begin
    Result := FxComponent.DBSearchFields.Items[Row];
  end;
end;

procedure TFormDBSearchEditor.GridDrawCell(Sender: TObject; Col,
  Row: Integer; Rect: TRect; State: TGridDrawState);
var
  CellText: string;
  DBSearchField: TxDBSearchField;
begin
  CellText := '';
  if (gdFixed in State) then begin
    case Col of
      0: CellText := 'DBSearchField name';
    end;
  end else begin
    DBSearchField := xItemByRow(Row - 1);
    if DBSearchField <> nil then begin
      case Col of
        0: CellText := DBSearchField.Name;
      end;
    end;
  end;
  DrawCellText(Grid, Col, Row, CellText, Rect, taLeftJustify, vaCenter);
end;

procedure TFormDBSearchEditor.GridSelectCell(Sender: TObject; Col,
  Row: Integer; var CanSelect: Boolean);
begin
  SelectxItem(xItemByRow(Row - 1));
end;


procedure TFormDBSearchEditor.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TFormDBSearchEditor.BtnDeleteClick(Sender: TObject);
var
  DBSearchField: TxDBSearchField;
begin
  //    "".

  //    :
  if (cbConfirmDelete.Checked = True)
  and (Application.MessageBox('Delete item ?', 'Confirmation',
    mb_YesNo + mb_IconQuestion ) <> ID_YES)
  then Exit;

  //  :
  DBSearchField := xItemByRow(Grid.Row - 1);
  if DBSearchField <> nil then begin
    Designer.ValidateRename(DBSearchField, DBSearchField.Name, '');
    FDeleting := True;
    try
      DBSearchField.Free;
      Designer.Modified;
    finally
      FDeleting := False;
    end;
  end;
end;


procedure TFormDBSearchEditor.GridKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  //   .
  if Shift = [] then begin
    case Key of
      VK_RETURN:
        if xItemByRow(Grid.Row - 1) <> nil then begin
          ActivateInspector(#0);
        end;
      VK_DELETE:
        BtnDeleteClick(nil);
    end;
  end;
end;

procedure TFormDBSearchEditor.BtnAddClick(Sender: TObject);
var
  DBSearchField: TxDBSearchField;
begin
  //     .
  if not CheckComponent then Exit;
  DBSearchField := TxDBSearchField.Create( xComponent.Owner );
  if DBSearchField <> nil then
    DBSearchField.Name := UniqueName( DBSearchField );
  xComponent.Add(DBSearchField);
  UpdateData;
  SelectxItem(DBSearchField);
end;

procedure TFormDBSearchEditor.BtnClearClick(Sender: TObject);
begin
  //    " " (   -
  //  ).

  //    :
  if (Application.MessageBox('Delete all items ?', 'Confirmation',
    mb_YesNo + mb_IconQuestion ) <> ID_YES)
  then Exit;

  //   :
  if xComponent <> nil then begin
    xComponent.Clear;
    Designer.Modified;
    UpdateData;
  end;
end;

end.

